@@ -8,6 +8,7 @@ from packaging.version import parse
 extensions = [
     "sphinx.ext.autodoc",
     "sphinx.ext.intersphinx",
+    "sphinx_tabs.tabs",
     "sphinx_autodoc_typehints",
     "sphinx_rtd_theme",
 ]
@@ -28,7 +29,7 @@ language = "en"
 exclude_patterns = ["_build"]
 pygments_style = "sphinx"
 autodoc_default_options = {"members": True, "show-inheritance": True}
-autodoc_mock_imports = ["_typeshed"]
+autodoc_mock_imports = ["_typeshed", "pytest", "_pytest"]
 todo_include_todos = False
 
 html_theme = "sphinx_rtd_theme"
