@@ -1,15 +1,12 @@
+from __future__ import annotations
+
 import enum
-import sys
 from dataclasses import dataclass
-from typing import Any, Dict, Generic, Set, TypeVar, Union, overload
+from typing import Any, Generic, Literal, TypeVar, final, overload
 from weakref import WeakKeyDictionary
 
-from ._core._eventloop import get_asynclib
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from ._core._eventloop import get_async_backend
+from .abc import AsyncBackend
 
 T = TypeVar("T")
 D = TypeVar("D")
@@ -24,10 +21,11 @@ async def checkpoint() -> None:
         await checkpoint_if_cancelled()
         await cancel_shielded_checkpoint()
 
+
     .. versionadded:: 3.0
 
     """
-    await get_asynclib().checkpoint()
+    await get_async_backend().checkpoint()
 
 
 async def checkpoint_if_cancelled() -> None:
@@ -39,7 +37,7 @@ async def checkpoint_if_cancelled() -> None:
     .. versionadded:: 3.0
 
     """
-    await get_asynclib().checkpoint_if_cancelled()
+    await get_async_backend().checkpoint_if_cancelled()
 
 
 async def cancel_shielded_checkpoint() -> None:
@@ -51,25 +49,40 @@ async def cancel_shielded_checkpoint() -> None:
         with CancelScope(shield=True):
             await checkpoint()
 
+
     .. versionadded:: 3.0
 
     """
-    await get_asynclib().cancel_shielded_checkpoint()
+    await get_async_backend().cancel_shielded_checkpoint()
 
 
-def current_token() -> object:
-    """Return a backend specific token object that can be used to get back to the event loop."""
-    return get_asynclib().current_token()
+@final
+@dataclass(frozen=True, repr=False)
+class EventLoopToken:
+    """
+    An opaque object that holds a reference to an event loop.
+
+    .. versionadded:: 4.11.0
+    """
+
+    backend_class: type[AsyncBackend]
+    native_token: object
 
 
-_run_vars = WeakKeyDictionary()  # type: WeakKeyDictionary[Any, Dict[str, Any]]
-_token_wrappers: Dict[Any, "_TokenWrapper"] = {}
+def current_token() -> EventLoopToken:
+    """
+    Return a token object that can be used to call code in the current event loop from
+    another thread.
+
+    .. versionadded:: 4.11.0
+
+    """
+    backend_class = get_async_backend()
+    raw_token = backend_class.current_token()
+    return EventLoopToken(backend_class, raw_token)
 
 
-@dataclass(frozen=True)
-class _TokenWrapper:
-    __slots__ = "_token", "__weakref__"
-    _token: object
+_run_vars: WeakKeyDictionary[object, dict[RunVar[Any], Any]] = WeakKeyDictionary()
 
 
 class _NoValueSet(enum.Enum):
@@ -79,60 +92,47 @@ class _NoValueSet(enum.Enum):
 class RunvarToken(Generic[T]):
     __slots__ = "_var", "_value", "_redeemed"
 
-    def __init__(
-        self, var: "RunVar[T]", value: Union[T, Literal[_NoValueSet.NO_VALUE_SET]]
-    ):
+    def __init__(self, var: RunVar[T], value: T | Literal[_NoValueSet.NO_VALUE_SET]):
         self._var = var
-        self._value: Union[T, Literal[_NoValueSet.NO_VALUE_SET]] = value
+        self._value: T | Literal[_NoValueSet.NO_VALUE_SET] = value
         self._redeemed = False
 
 
 class RunVar(Generic[T]):
-    """Like a :class:`~contextvars.ContextVar`, expect scoped to the running event loop."""
+    """
+    Like a :class:`~contextvars.ContextVar`, except scoped to the running event loop.
+    """
 
     __slots__ = "_name", "_default"
 
     NO_VALUE_SET: Literal[_NoValueSet.NO_VALUE_SET] = _NoValueSet.NO_VALUE_SET
 
-    _token_wrappers: Set[_TokenWrapper] = set()
-
     def __init__(
-        self,
-        name: str,
-        default: Union[T, Literal[_NoValueSet.NO_VALUE_SET]] = NO_VALUE_SET,
+        self, name: str, default: T | Literal[_NoValueSet.NO_VALUE_SET] = NO_VALUE_SET
     ):
         self._name = name
         self._default = default
 
     @property
-    def _current_vars(self) -> Dict[str, T]:
-        token = current_token()
-        while True:
-            try:
-                return _run_vars[token]
-            except TypeError:
-                # Happens when token isn't weak referable (TrioToken).
-                # This workaround does mean that some memory will leak on Trio until the problem
-                # is fixed on their end.
-                token = _TokenWrapper(token)
-                self._token_wrappers.add(token)
-            except KeyError:
-                run_vars = _run_vars[token] = {}
-                return run_vars
+    def _current_vars(self) -> dict[RunVar[T], T]:
+        native_token = current_token().native_token
+        try:
+            return _run_vars[native_token]
+        except KeyError:
+            run_vars = _run_vars[native_token] = {}
+            return run_vars
 
     @overload
-    def get(self, default: D) -> Union[T, D]:
-        ...
+    def get(self, default: D) -> T | D: ...
 
     @overload
-    def get(self) -> T:
-        ...
+    def get(self) -> T: ...
 
     def get(
-        self, default: Union[D, Literal[_NoValueSet.NO_VALUE_SET]] = NO_VALUE_SET
-    ) -> Union[T, D]:
+        self, default: D | Literal[_NoValueSet.NO_VALUE_SET] = NO_VALUE_SET
+    ) -> T | D:
         try:
-            return self._current_vars[self._name]
+            return self._current_vars[self]
         except KeyError:
             if default is not RunVar.NO_VALUE_SET:
                 return default
@@ -145,8 +145,8 @@ class RunVar(Generic[T]):
 
     def set(self, value: T) -> RunvarToken[T]:
         current_vars = self._current_vars
-        token = RunvarToken(self, current_vars.get(self._name, RunVar.NO_VALUE_SET))
-        current_vars[self._name] = value
+        token = RunvarToken(self, current_vars.get(self, RunVar.NO_VALUE_SET))
+        current_vars[self] = value
         return token
 
     def reset(self, token: RunvarToken[T]) -> None:
@@ -158,11 +158,11 @@ class RunVar(Generic[T]):
 
         if token._value is _NoValueSet.NO_VALUE_SET:
             try:
-                del self._current_vars[self._name]
+                del self._current_vars[self]
             except KeyError:
                 pass
         else:
-            self._current_vars[self._name] = token._value
+            self._current_vars[self] = token._value
 
         token._redeemed = True
 
