@@ -3,9 +3,9 @@ from __future__ import annotations
 import asyncio
 import ssl
 import sys
-from collections.abc import Generator
+from collections.abc import Generator, Iterator
 from ssl import SSLContext
-from typing import Any
+from typing import TYPE_CHECKING, Any
 from unittest.mock import Mock
 
 import pytest
@@ -13,6 +13,9 @@ import trustme
 from _pytest.fixtures import SubRequest
 from trustme import CA
 
+if TYPE_CHECKING:
+    from blockbuster import BlockBuster
+
 uvloop_marks = []
 try:
     import uvloop
@@ -52,6 +55,34 @@ if sys.version_info >= (3, 12):
     )
 
 
+@pytest.fixture(autouse=True)
+def blockbuster() -> Iterator[BlockBuster | None]:
+    try:
+        from blockbuster import blockbuster_ctx
+    except ImportError:
+        yield None
+        return
+
+    with blockbuster_ctx(
+        "anyio", excluded_modules=["anyio.pytest_plugin", "anyio._backends._asyncio"]
+    ) as bb:
+        bb.functions["socket.socket.accept"].can_block_in(
+            "anyio/_core/_asyncio_selector_thread.py", {"get_selector"}
+        )
+        for func in ["os.stat", "os.unlink"]:
+            bb.functions[func].can_block_in(
+                "anyio/_core/_sockets.py", "setup_unix_local_socket"
+            )
+
+        yield bb
+
+
+@pytest.fixture
+def deactivate_blockbuster(blockbuster: BlockBuster | None) -> None:
+    if blockbuster is not None:
+        blockbuster.deactivate()
+
+
 @pytest.fixture(params=[*asyncio_params, pytest.param("trio")])
 def anyio_backend(request: SubRequest) -> tuple[str, dict[str, Any]]:
     return request.param
@@ -84,8 +115,32 @@ def client_context(ca: CA) -> SSLContext:
 
 @pytest.fixture
 def asyncio_event_loop() -> Generator[asyncio.AbstractEventLoop, None, None]:
-    loop = asyncio.DefaultEventLoopPolicy().new_event_loop()
-    asyncio.set_event_loop(loop)
+    if sys.version_info >= (3, 13):
+        loop = asyncio.EventLoop()
+    else:
+        loop = asyncio.new_event_loop()
+
+    if sys.version_info < (3, 10):
+        asyncio.set_event_loop(loop)
+
     yield loop
-    asyncio.set_event_loop(None)
+
+    if sys.version_info < (3, 10):
+        asyncio.set_event_loop(None)
+
     loop.close()
+
+
+if sys.version_info >= (3, 14):
+
+    def no_other_refs() -> list[object]:
+        return [sys._getframe(1).f_generator]
+
+elif sys.version_info >= (3, 11):
+
+    def no_other_refs() -> list[object]:
+        return []
+else:
+
+    def no_other_refs() -> list[object]:
+        return [sys._getframe(1)]
