@@ -12,8 +12,6 @@ from anyio.lowlevel import (
     checkpoint_if_cancelled,
 )
 
-pytestmark = pytest.mark.anyio
-
 
 @pytest.mark.parametrize("cancel", [False, True])
 async def test_checkpoint_if_cancelled(cancel: bool) -> None:
@@ -126,3 +124,16 @@ class TestRunVar:
         var.reset(token)
         with pytest.raises(ValueError, match="This token has already been used"):
             var.reset(token)
+
+    async def test_runvar_does_not_share_storage_by_name(self) -> None:
+        var1: RunVar[int] = RunVar("var", 1)
+        var2: RunVar[str] = RunVar("var", "a")
+
+        assert var1.get() == 1
+        assert var2.get() == "a"
+
+        var1.set(2)
+        var2.set("b")
+
+        assert var1.get() == 2
+        assert var2.get() == "b"
