@@ -10,7 +10,6 @@ import pytest
 from anyio import create_task_group, fail_after, open_signal_receiver, to_thread
 
 pytestmark = [
-    pytest.mark.anyio,
     pytest.mark.skipif(
         sys.platform == "win32",
         reason="Signal delivery cannot be tested on Windows",
@@ -35,7 +34,7 @@ async def test_receive_signals() -> None:
 async def test_task_group_cancellation_open() -> None:
     async def signal_handler() -> None:
         with open_signal_receiver(signal.SIGUSR1) as sigiter:
-            async for v in sigiter:
+            async for _ in sigiter:
                 pytest.fail("SIGUSR1 should not be sent")
 
             pytest.fail("signal_handler should have been cancelled")
@@ -49,7 +48,7 @@ async def test_task_group_cancellation_open() -> None:
 
 async def test_task_group_cancellation_consume() -> None:
     async def consume(sigiter: AsyncIterable[int]) -> None:
-        async for v in sigiter:
+        async for _ in sigiter:
             pytest.fail("SIGUSR1 should not be sent")
 
         pytest.fail("consume should have been cancelled")
