@@ -19,8 +19,6 @@ from anyio import (
 )
 from anyio.abc import Process
 
-pytestmark = pytest.mark.anyio
-
 
 async def test_run_sync_in_process_pool() -> None:
     """
@@ -73,6 +71,7 @@ async def test_cancel_before() -> None:
     pytest.raises(LookupError, to_process._process_pool_workers.get)
 
 
+@pytest.mark.usefixtures("deactivate_blockbuster")
 async def test_cancel_during() -> None:
     """
     Test that cancelling an operation on the worker process causes the process to be
